"""This module defines variables for the package metadata."""

NAME: str = "picharsso"
"""The package name."""

DESCRIPTION: str = "A utility for converting images to text art."
"""The package description."""

VERSION: str = "2.0.1"
"""The current package version.
"""

REPO_URL: str = f"https://github.com/kelvindecosta/{NAME}"
"""The URL of the package source code."""

DOCS_URL: str = f"https://kelvindecosta.github.io/{NAME}"
"""The URL of the package documentation."""

AUTHOR: str = "Kelvin DeCosta"
"""The package author."""

LICENSE: str = "MIT"
"""The package license."""
